/*
 * Copyright (c) 1995: Open Software Foundation Inc.
 *
 * All Rights Reserved. Please see COPYRIGHT.DCEWEB
 * in the top-level directory of this release.
 */
/* ______________________________________________________________________ */
/*
 * @OSF_COPYRIGHT@
 * Copyright (c) 1994 Open Software Foundation, Inc.
 * ALL RIGHTS RESERVED (DCE).  See the file named COPYRIGHT.DCE in the
 * src directory for the full copyright text.
 */
/*
 * HISTORY
 * $Log: aclconv.c,v $
 * Revision 1.1.2.8  1994/07/15  17:27:04  bowe
 * 	Remove dce_acl_g_helpstring. [CR 11308]
 * 	[1994/07/15  17:26:20  bowe]
 *
 * Revision 1.1.2.7  1994/06/10  22:26:13  cbrooks
 * 	CR10897 - only memset() sec_acl_entries once in incr__acl_ent_list
 * 	[1994/06/10  22:25:12  cbrooks]
 * 
 * 	CR 10897
 * 	[1994/06/10  21:11:31  cbrooks]
 * 
 * Revision 1.1.2.6  1994/06/03  13:59:46  bowe
 * 	Use rpc_sm_allocate()/rpc_sm_client_free() instead of malloc()/free().
 * 	[CR 10778]
 * 
 * Revision 1.1.2.5  1994/05/18  19:27:59  bowe
 * 	Check for NULL ptrs in dce_acl_inq_prin_and_group()
 * 	[1994/05/16  16:59:52  bowe]
 * 
 * Revision 1.1.2.4  1994/05/10  21:45:00  bowe
 * 	Adjust for obj header simplification
 * 	[1994/05/10  21:40:01  bowe]
 * 
 * Revision 1.1.2.3  1994/05/05  15:18:53  bowe
 * 	Renamed dce_acl_obj_init(), dce_acl_obj_free_entries() (new names).
 * 	Added some missing cases for delegation in _free_entries.  [CR 10001]
 * 	[1994/05/05  13:32:11  bowe]
 * 
 * Revision 1.1.2.2  1994/04/27  20:00:16  bowe
 * 	Check for NULL args in dce_acl_inq_prin_and_group().
 * 	[1994/04/27  19:59:58  bowe]
 * 
 * Revision 1.1.2.1  1994/04/26  19:11:25  bowe
 * 	Initial checkin (was part of dceaclimpl.c, now defunct)
 * 	[1994/04/26  19:09:18  bowe]
 * 
 * $EndLog$
 */
/* ______________________________________________________________________ */

/*  ACL convenience functions.  */

#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>

#include <aclimpl.h>

#include <dce/pgo.h>
#include <dce/secidmap.h>
#include <dce/dce_cf.h>

/* ______________________________________________________________________ */
/* Generic printstring for regular/basic/simple ACL manager.  Programmers
 * must define their own helpstring (which contains manager name).
 */

sec_acl_printstring_t dce_acl_g_printstring[] = {
    { "r",  "read",	sec_acl_perm_read	},
    { "w",  "write",	sec_acl_perm_write	},
    { "x",  "execute",	sec_acl_perm_execute	},
    { "c",  "control",	sec_acl_perm_control	},
    { "i",  "insert",	sec_acl_perm_insert	},
    { "d",  "delete",	sec_acl_perm_delete	},
    { "t",  "test",	sec_acl_perm_test	}
};

/* ______________________________________________________________________ */
/*  Create an ACL for an object of type mgr_type.  Caller passes in
 *  the (pointer to) ACL itself.  This will initialize the fields.
 */

void
dce_acl_obj_init(
    uuid_t		*mgr_type,
    sec_acl_t		*acl,
    error_status_t	*st
)
{
    unsigned_char_t	*cellname;
    sec_rgy_handle_t	rh;
    *st = error_status_ok;
    acl->sec_acl_manager_type	= *mgr_type;
    acl->num_entries		= 0;
    acl->sec_acl_entries	= NULL;

    /* First, null out the ACL structure. */
    memset(&acl->default_realm, 0, sizeof(sec_acl_id_t));

    /* Get the realm of the ACL */
    dce_cf_get_cell_name((char **)&cellname, st);
    STAT_CHECK_RET(*st);
    sec_rgy_site_open(cellname, &rh, st);
    if (*st != error_status_ok) {
	free(cellname);
	return;
    }
    sec_id_parse_name(rh, cellname, NULL, &acl->default_realm.uuid, NULL,
	NULL, st);
    free(cellname);
    STAT_CHECK_RET(*st);
    sec_rgy_site_close(rh, st);
    acl->default_realm.name	= NULL;
}

/* ______________________________________________________________________ */
/*  Free space used by an ACL (the entries), then NULL it out. */

void
dce_acl_obj_free_entries(
    sec_acl_t		*acl,
    error_status_t	*st
)
{
    int			i;
    sec_acl_entry_t	*ae;

    *st = error_status_ok;

    for (i=0; i<acl->num_entries; i++) {
	ae = &acl->sec_acl_entries[i];
	/* free each entry */

	switch (ae->entry_info.entry_type) {
	case sec_acl_e_type_user:
	case sec_acl_e_type_group:
	case sec_acl_e_type_foreign_other:
	    /* in most cases, names are null (uuid's are used only) */
	    if (ae->entry_info.tagged_union.id.name)
		rpc_sm_client_free(ae->entry_info.tagged_union.id.name, st);
	    break;

	case sec_acl_e_type_foreign_user:
	case sec_acl_e_type_foreign_group:
	    /* in most cases, names are null (uuid's are used only) */

	    if (ae->entry_info.tagged_union.foreign_id.id.name)
		rpc_sm_client_free(ae->entry_info.tagged_union.foreign_id.id.name, 
				   st);
	    if (ae->entry_info.tagged_union.foreign_id.realm.name)
		rpc_sm_client_free(ae->entry_info.tagged_union.foreign_id.realm.name, st);
	    break;

	default:
	    break;
	}
    }

    if (acl->sec_acl_entries)
	rpc_sm_client_free(acl->sec_acl_entries, st);
    memset(acl, 0, sizeof(sec_acl_t));
    return ;
}

/* ______________________________________________________________________ */
/*  Routines for adding entries to ACLs. */

/*  Increase the size of the entry list by 1.  Does malloc() or realloc().
 *  Caller calls dce_acl_obj_free_entries() to free all the stuff for this ACL.
 */

static sec_acl_entry_t *
incr__acl_ent_list(
    sec_acl_t		*acl
)
{
    sec_acl_entry_t		*e, *ep, *ep2;
    unsigned32			i;
    error_status_t		st;

    if (acl->num_entries == 0) {
	acl->sec_acl_entries = rpc_sm_allocate(sizeof(sec_acl_entry_t), &st);
	if (st != error_status_ok)
	    return NULL;
	acl->num_entries = 1;
    }
    else {
	/* Allocate a new list, then copy entries.  If successful,
	 * then set sec_acl_entries within ACL struct. */

	e = rpc_sm_allocate((acl->num_entries + 1) * sizeof *e, &st);
	if (st != error_status_ok)
	    return NULL;

	for (i=0,ep=e,ep2=acl->sec_acl_entries; i<acl->num_entries; i++) 
	{
	    *ep++ = *ep2++;
	}

	rpc_sm_client_free(acl->sec_acl_entries, &st);
	if (st != error_status_ok)
	    return NULL;
	acl->sec_acl_entries = e;
	acl->num_entries++;
    }

    ep2 = &acl->sec_acl_entries[acl->num_entries-1];
    memset(ep2, 0, sizeof(sec_acl_entry_t));
    return ep2 ;
}

static void
dce_acl__check_perm_bits(
    sec_acl_t			*acl,
    sec_acl_permset_t		permset,
    error_status_t		*st
)
{
    acl_mgt_t			*am_info;

    /* Get max permission bits for this  */
    am_info = dce_acl__get_info_from_type(&acl->sec_acl_manager_type, st);
    STAT_CHECK_RET(*st);

    if ( (~am_info->max_perm) & permset)
	*st = sec_acl_invalid_permission;
}


/*  Add an ACL entry (user/grp, domestic/foreign) to the given ACL. */

void
dce_acl_obj_add_id_entry(
    sec_acl_t			*acl,
    sec_acl_entry_type_t	entry_type,
    sec_acl_permset_t		permset,
    uuid_t			*id,
    error_status_t		*st
)
{
    sec_acl_entry_t		*e = 0 ;

    *st = error_status_ok;

    switch (entry_type) {
    case sec_acl_e_type_user:
    case sec_acl_e_type_group:
    case sec_acl_e_type_foreign_other:
	dce_acl__check_perm_bits(acl, permset, st);
	STAT_CHECK_RET(*st);
	if ((e = incr__acl_ent_list(acl)) == NULL)
	{
	    *st = rpc_s_no_memory ;
	    return;
	}
	e->perms			= permset;
	e->entry_info.entry_type	= entry_type;

	/* Just the ID */

	e->entry_info.tagged_union.id.uuid	= *id;
	e->entry_info.tagged_union.id.name	= NULL;
	break;

    default:
	*st = sec_acl_invalid_entry_type;
	break;
    }
}

/*  Add a user ACL entry to the given ACL. */

void
dce_acl_obj_add_user_entry(
    sec_acl_t		*acl,
    sec_acl_permset_t	permset,
    uuid_t		*user,
    error_status_t	*st
)
{
    dce_acl_obj_add_id_entry(acl, sec_acl_e_type_user, permset, user, st);
}

/*  Add a group ACL entry to the given ACL. */

void
dce_acl_obj_add_group_entry(
    sec_acl_t		*acl,
    sec_acl_permset_t	permset,
    uuid_t		*group,
    error_status_t	*st
)
{
    dce_acl_obj_add_id_entry(acl, sec_acl_e_type_group, permset, group, st);
}

/*  Add an "obj" ACL entry to the given ACL. */

void
dce_acl_obj_add_obj_entry(
    sec_acl_t			*acl,
    sec_acl_entry_type_t	entry_type,
    sec_acl_permset_t		permset,
    error_status_t		*st
)
{
    sec_acl_entry_t		*e = 0 ;

    *st = error_status_ok;
    if ((e = incr__acl_ent_list(acl)) == NULL)
    {
	*st = rpc_s_no_memory ;
	return;
    }

    switch (entry_type) {
    case sec_acl_e_type_unauthenticated:
    case sec_acl_e_type_any_other:

	dce_acl__check_perm_bits(acl, permset, st);
	STAT_CHECK_RET(*st);
	e->perms			= permset;
	e->entry_info.entry_type	= entry_type;
	break;

    default:
	*st = sec_acl_invalid_entry_type;
	break;
    }
}

/*  Add ACL entry for "unauthenticated" to the given ACL. */

void
dce_acl_obj_add_unauth_entry(
    sec_acl_t		*acl,
    sec_acl_permset_t	permset,
    error_status_t	*st
)
{
    dce_acl_obj_add_obj_entry(acl, sec_acl_e_type_unauthenticated, permset, st);
}

/*  Add ACL entry for "any_other" to the given ACL. */

void
dce_acl_obj_add_any_other_entry(
    sec_acl_t		*acl,
    sec_acl_permset_t	permset,
    error_status_t	*st
)
{
    dce_acl_obj_add_obj_entry(acl, sec_acl_e_type_any_other, permset, st);
}

/*  Add ACL entry for a foreign user or group to the given ACL. */

void
dce_acl_obj_add_foreign_entry(
    sec_acl_t			*acl,
    sec_acl_entry_type_t	entry_type,
    sec_acl_permset_t		permset,
    uuid_t			*realm,
    uuid_t			*id,
    error_status_t		*st
)
{
    sec_acl_entry_t		*e;
    sec_id_foreign_t		*fid;

    *st = error_status_ok;
    switch (entry_type) {
    case sec_acl_e_type_foreign_user:
    case sec_acl_e_type_foreign_group:
	dce_acl__check_perm_bits(acl, permset, st);
	STAT_CHECK_RET(*st);
	if((e = incr__acl_ent_list(acl)) == NULL)
	{
	    *st = rpc_s_no_memory ;
	    return;
	}
	
	e->perms			= permset;
	e->entry_info.entry_type	= entry_type;

	fid = &e->entry_info.tagged_union.foreign_id;
	fid->realm.uuid = *realm;
	fid->id.uuid    = *id;
	fid->realm.name = fid->id.name = NULL;
	break;

    default:
	*st = sec_acl_invalid_entry_type;
	break;
    }
}

/* ______________________________________________________________________ */
/* Copy an ACL: acl1 -> acl2.
 * Space for acl2 is passed in, but the sec_acl_entries are allocated
 *  (just call free(acl2->sec_acl_entries) when done with it).
 */
void
dce_acl_copy_acl(
    sec_acl_t		*acl1,
    sec_acl_t		*acl2,
    error_status_t	*st
)
{
    int		i;
    idl_char	*cp;
    sec_acl_entry_t		*ae1, *ae2;

    acl2->default_realm		= acl1->default_realm;
    acl2->sec_acl_manager_type	= acl1->sec_acl_manager_type;
    acl2->sec_acl_entries	= (sec_acl_entry_t *)rpc_sm_allocate(
		sizeof(sec_acl_entry_t)*acl1->num_entries, st);
    STAT_CHECK_RET(*st);

    for (i=0; i<acl1->num_entries; i++) {
	ae1 = &acl1->sec_acl_entries[i];
	ae2 = &acl2->sec_acl_entries[i];
	acl2->sec_acl_entries[i] = acl1->sec_acl_entries[i];

	switch (ae1->entry_info.entry_type) {
	case sec_acl_e_type_user:
	case sec_acl_e_type_group:
	case sec_acl_e_type_foreign_other:
	    ae2->entry_info.tagged_union.id.uuid =
		ae1->entry_info.tagged_union.id.uuid;
	    if (ae1->entry_info.tagged_union.id.name) {
		cp = ae1->entry_info.tagged_union.id.name;
		ae2->entry_info.tagged_union.id.name =
			(idl_char *)rpc_sm_allocate(strlen((char *)cp) + 1, st);
		strcpy((char *)ae2->entry_info.tagged_union.id.name, (char *)cp);
	    }
	    else
		ae2->entry_info.tagged_union.id.name = NULL;
	    break;

	case sec_acl_e_type_mask_obj:
	case sec_acl_e_type_user_obj:
	case sec_acl_e_type_group_obj:
	case sec_acl_e_type_other_obj:
	case sec_acl_e_type_unauthenticated:
	case sec_acl_e_type_any_other:
	    break;			/* no action */

	case sec_acl_e_type_foreign_user:
	case sec_acl_e_type_foreign_group:
	    ae2->entry_info.tagged_union.foreign_id =
		ae1->entry_info.tagged_union.foreign_id;
	    break;
	case sec_acl_e_type_extended:
	    ae2->entry_info.tagged_union.extended_info =
		ae1->entry_info.tagged_union.extended_info;
	    /* ae2->entry_info.tagged_union.extended_info.pickled_data
	     * is NOT a pointer, so we cannot allocate a copy.
	     */
	    break;
	default:
	    break;
	}
    }
    acl2->num_entries = acl1->num_entries;
}

/* ______________________________________________________________________ */
/*  Retrieve the UUID of the appropriate ACL object from a standard
 *  database header.
 */

void
dce_acl_inq_acl_from_header(
    dce_db_header_t	dbh,
    sec_acl_type_t	sec_acl_type,
    uuid_t		*acl_uuid,
    error_status_t	*st
)
{
    dce_db_dataheader_t	*dhdr;

    *st = error_status_ok;
    dhdr = &dbh.tagged_union.h;

    switch (sec_acl_type) {
    default:
	*st = sec_acl_invalid_acl_type;
	break;

    case sec_acl_type_object:
	*acl_uuid = dhdr->acl_uuid;
	break;

    case sec_acl_type_default_object:
	*acl_uuid = dhdr->def_object_acl;
	break;

    case sec_acl_type_default_container:
	*acl_uuid = dhdr->def_container_acl;
	break;
    }
}

/* ______________________________________________________________________ */
/* Find the principal and group of the caller.  This is useful for filling
 * in the owner_id and group_id fields of standard data/obj headers.
 */
/* removed creds and used pacs - Mez */

void
dce_acl_inq_prin_and_group(
    handle_t		h,
    uuid_t		*principal,
    uuid_t		*group,
    error_status_t	*st
)
{
    sec_id_pac_t                *pac_p;

    *st = error_status_ok;

    if (!h) {
	if (principal) uuid_create_nil(principal, st);
	if (group)     uuid_create_nil(group, st);
	return;
    }
    /* Get the caller's pac */
    dce_acl_inq_client_pac(h, &pac_p, st);
    STAT_CHECK_RET(*st);
    if (dce_acl_is_unauthenticated(pac_p)) {
	if (principal)	uuid_create_nil(principal, st);
	if (group)	uuid_create_nil(group, st);
	return;
    }
    if (principal)	*principal = pac_p->principal.uuid;
    if (group)		*group     = pac_p->group.uuid;
}

/* ______________________________________________________________________ */
