/*
 * Copyright (c) 1995: Open Software Foundation Inc.
 *
 * All Rights Reserved. Please see COPYRIGHT.DCEWEB
 * in the top-level directory of this release.
 */
/* ______________________________________________________________________ */
/*
 * @OSF_COPYRIGHT@
 * Copyright (c) 1994 Open Software Foundation, Inc.
 * ALL RIGHTS RESERVED (DCE).  See the file named COPYRIGHT.DCE in the
 * src directory for the full copyright text.
 */
/*
 * HISTORY
 * $Log: acleval.c,v $
 * Revision 1.1.2.6  1994/09/20  17:34:45  bowe
 * 	Free cred cursor when done with it. [CR 12255]
 * 	[1994/09/20  17:34:26  bowe]
 *
 * Revision 1.1.2.5  1994/09/15  14:37:07  bowe
 * 	NULL for unused returns from rpc_binding_inq_auth_caller() [CR 12196]
 * 	[1994/09/15  14:36:53  bowe]
 * 
 * Revision 1.1.2.4  1994/07/26  21:07:58  bowe
 * 	Set check_groups flag for sec_acl_e_type_group_obj and its _deleg.
 * 	Removed some unused local variables.  [CR 11378]
 * 	[1994/07/26  21:07:43  bowe]
 * 
 * Revision 1.1.2.3  1994/06/06  15:07:07  rsalz
 * 	Treat NIL UUID as local realm.
 * 	[1994/06/06  15:07:01  rsalz]
 * 
 * Revision 1.1.2.2  1994/05/18  19:17:40  bowe
 * 	Init some pointers in dce_acl__inq_unauthen_perms() [10657]
 * 	[1994/05/16  16:01:21  bowe]
 * 
 * Revision 1.1.2.1  1994/04/26  19:11:26  bowe
 * 	Initial checkin (was part of dceaclimpl.c, now defunct)
 * 	[1994/04/26  19:09:26  bowe]
 * 
 * $EndLog$
 */
/* ______________________________________________________________________ */

#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>

#include <aclimpl.h>

/* ______________________________________________________________________ */
/*
 * Macro to mask entry permset against mask_obj if necessary and return
 * the result of that operation.  Note that this macro was ultimately derived
 * from the DEC Security private routine "access_granted" and should track the
 * intended functionality of that routine.  Note also that the choice of ACLE
 * types in the lists below is based on the definition of MASK_OBJ and not on
 * the application requirements.
 */

#define SAET_no_mask_obj	\
    (	(1 << sec_acl_e_type_user_obj) | \
	(1 << sec_acl_e_type_other_obj) | \
	(1 << sec_acl_e_type_mask_obj) | \
	(1 << sec_acl_e_type_unauthenticated) )

#define SAET_use_mask_obj	\
    (	(1 << sec_acl_e_type_group_obj) | \
	(1 << sec_acl_e_type_any_other) | \
	(1 << sec_acl_e_type_user) | \
	(1 << sec_acl_e_type_group) | \
	(1 << sec_acl_e_type_foreign_other) | \
	(1 << sec_acl_e_type_foreign_user) | \
	(1 << sec_acl_e_type_foreign_group) | \
	(1 << sec_acl_e_type_extended) )

#define MASKIT(ep, mask_perms) ( \
	    (1 << (int)ep->entry_info.entry_type) & SAET_no_mask_obj \
	    ?  ep->perms \
	    :  (1 << (int)ep->entry_info.entry_type) & SAET_use_mask_obj \
		?  ep->perms & mask_perms : dce_acl_c_no_permissions)

#define SAME_FOREIGN_REALM(entry) \
    uuid_equal(&pac_p->realm.uuid, &ep->entry_info.tagged_union.foreign_id.realm.uuid, st)

/* ______________________________________________________________________ */
/*  Checks whether client's credentials are authenticated and that they
 *  grant the desired permissions.
 */
void
dce_acl_is_client_authorized(
  /* [in] */
    handle_t		h,
    uuid_t		*mgr_type,
    uuid_t		*acl_uuid,
    uuid_t		*own_id,
    uuid_t		*grp_id,
    sec_acl_permset_t	desired_perms,
  /* [out] */
    boolean32		*authorized,
    error_status_t	*st
)
{
    sec_acl_permset_t	perms;

    *authorized = FALSE;
    /* Get ACL for this object/mgr type */
    dce_acl_inq_client_permset(h, mgr_type, acl_uuid, own_id, grp_id, &perms, st);
    STAT_CHECK_RET(*st);

    if ( (desired_perms & perms) == desired_perms )
	*authorized = TRUE;
}

/* ______________________________________________________________________ */
/*  Checks whether client's pac is authenticated and that it
 *  grants the desired permissions.
 */
void
dce_acl_is_pac_authorized(
  /* [in] */
    sec_id_pac_t	*pac_p,
    uuid_t		*mgr_type,
    uuid_t		*acl_uuid,
    uuid_t		*own_id,
    uuid_t		*grp_id,
    sec_acl_permset_t	desired_perms,
  /* [out] */
    boolean32		*authorized,
    error_status_t	*st
)
{
    sec_acl_permset_t	perms;

    *authorized = FALSE;
    /* Get ACL for this object/mgr type */
    dce_acl_inq_pac_permset(pac_p, mgr_type, acl_uuid, own_id, grp_id, &perms, st);
    STAT_CHECK_RET(*st);

    if ( (desired_perms & perms) == desired_perms )
	*authorized = TRUE;
}

/* ______________________________________________________________________ */
/*  Checks whether credentials are those of "unauthenticated".
 *  We declare the caller unauthenticated when rpc_binding_inq_auth_caller()
 *  says no_authen.
 */
/* no creds - Mez
*static int			s_unauthen_cred_int;
*static rpc_authz_cred_handle_t	s_unauthen_cred_h = { 0, &s_unauthen_cred_int };
*/

boolean32
dce_acl_is_unauthenticated(
/* no creds - Mez
*    rpc_authz_cred_handle_t	cred_h
*/
    sec_id_pac_t    *pac_p
)
{
/*    return (cred_h.data == &s_unauthen_cred_int) ? TRUE : FALSE; */
      if (pac_p != NULL)
          return (!pac_p->authenticated);
      else
	{
	  return TRUE;
	}
}

/* ______________________________________________________________________ */
/*  Check what perms the client has, given the ACL.
 */
void
dce_acl_inq_client_permset(
  /* [in] */
    handle_t		h,
    uuid_t		*mgr_type,
    uuid_t		*acl_uuid,
    uuid_t		*own_id,
    uuid_t		*grp_id,
  /* [out] */
    sec_acl_permset_t	*permset,
    error_status_t	*st
)
{
/* no creds - Mez
*    rpc_authz_cred_handle_t	cred_h;
*/
    sec_id_pac_t                *pac_p;
    sec_acl_t			acl;
    acl_mgt_t			*am_info;

	unsigned_char_t *u_string;      /* For converting UUIDs to strings. */

    *st = error_status_ok;
    *permset = dce_acl_c_no_permissions;

    /* Get database handle */
    am_info = dce_acl__get_info_from_type(mgr_type, st);
    STAT_CHECK_RET(*st);

    /* Read acl from database.  Key is uuid of acl. */
    dce_db_fetch_by_uuid(am_info->db, acl_uuid, &acl, st);
    STAT_CHECK_RET(*st);

    /* check that registered mgr type matches ACL's mgr type */
    if (!uuid_equal(&am_info->mgr_type, &acl.sec_acl_manager_type, st)) {
	*st = acl_s_bad_manager_type;
	return;
    }

    /* Get the client's (caller's) credentials */
/* no creds - Mez
*    dce_acl_inq_client_creds(h, &cred_h, st);
*/
    dce_acl_inq_client_pac(h, &pac_p, st);
    STAT_CHECK_RET(*st);

    /* Compare ACL with PAC, determining permissions. */
/* no creds - Mez
*    dce_acl_inq_permset_for_creds(cred_h, &acl, own_id, grp_id,
*/
    dce_acl_inq_permset_for_pac(pac_p, &acl, own_id, grp_id,
	sec_acl_posix_no_semantics, permset, st);
}

/* ______________________________________________________________________ */
/*  Check what perms the client has, given the ACL.
 */
void
dce_acl_inq_pac_permset(
  /* [in] */
    sec_id_pac_t	*pac_p,
    uuid_t		*mgr_type,
    uuid_t		*acl_uuid,
    uuid_t		*own_id,
    uuid_t		*grp_id,
  /* [out] */
    sec_acl_permset_t	*permset,
    error_status_t	*st
)
{
/* no creds - Mez
*    rpc_authz_cred_handle_t	cred_h;
*/
    sec_acl_t			acl;
    acl_mgt_t			*am_info;

	unsigned_char_t *u_string;      /* For converting UUIDs to strings. */

    *st = error_status_ok;
    *permset = dce_acl_c_no_permissions;

    /* Get database handle */
    am_info = dce_acl__get_info_from_type(mgr_type, st);
    STAT_CHECK_RET(*st);

    /* Read acl from database.  Key is uuid of acl. */
    dce_db_fetch_by_uuid(am_info->db, acl_uuid, &acl, st);
    STAT_CHECK_RET(*st);

    /* check that registered mgr type matches ACL's mgr type */
    if (!uuid_equal(&am_info->mgr_type, &acl.sec_acl_manager_type, st)) {
	*st = acl_s_bad_manager_type;
	return;
    }

    /* Compare ACL with PAC, determining permissions. */
/* no creds - Mez
*    dce_acl_inq_permset_for_creds(cred_h, &acl, own_id, grp_id,
*/
    dce_acl_inq_permset_for_pac(pac_p, &acl, own_id, grp_id,
	sec_acl_posix_no_semantics, permset, st);
}

/* ______________________________________________________________________ */
/*  Return the client's PAC (Privilege Attribute Certificate).
 *  This is gotten from the RPC binding handle.
 */
/* Was dce_acl_inq_client_creds; changed to deal in pacs - Mez */

void
dce_acl_inq_client_pac(
  /* [in] */
    handle_t			h,
  /* [out] */
    sec_id_pac_t        	**pac_p,
    error_status_t		*st
)
{
    unsigned32			authz_svc;
    rpc_authz_handle_t          privs;

    *st = error_status_ok;
    *pac_p = NULL;	/* default case */

    /* Get caller's pac */
    rpc_binding_inq_auth_client(h, &privs, NULL, NULL, NULL, &authz_svc, st);

    if (*st == rpc_s_binding_has_no_auth) { 
	/* unauthenticated - pac_p is already NULL; just return. */
        /* do callers need to check for a null pac? Mez */
	*st = error_status_ok;
	return;
    }


    if (*st != rpc_s_ok)
	return;

    /* What kind of authen does client want? */
    if (authz_svc != rpc_c_authz_dce) {
	*st = rpc_s_authn_authz_mismatch;
	return;
    }

    /* Otherwise, there's a real pac we can use */
    *pac_p = (sec_id_pac_t*) privs;
    return;

/*    *cred_pa = sec_cred_get_initiator(rpc_creds, st);*/
}

/* ______________________________________________________________________ */

/*  This is an outline of "Extended Access Control Algorithm", which
 *  I got from the Security AES/DC (and the Delegation and EPAC spec).
 *	Check initiator:
 *	  Apply standard algorithm
 *	  If access denied, then deny access and return
 *	Check each intermediary (delegate): [all deleg removed - Mez]
 *	  For each extended PAC:
 *	    Apply standard algorithm, but also consider delegate ACLEs
 *	    If access denied, then deny access and return
 *	Grant access.
 *
 *  (A difference is that we need is to retrieve the permissions, not
 *  just say grant/deny. The grant/deny function will call the function
 *  that retrieves permissions, then compare.)
 *
 *  This is the order for matching in the standard algorithm.  In the
 *  extended (with delegates) algorithm the delegate match/tests come
 *  immediately after the corresponding "regular" entry tests.
 *	 1. USER_OBJ
 *	 2. USER
 *	 3. FOREIGN_USER
 *	 4. GROUP_OBJ
 *	 5. GROUP
 *	 6. FOREIGN_GROUP
 *	 7. OTHER_OBJ
 *	 8. FOREIGN_OTHER
 *	 9. ANY_OTHER
 *	10. UNAUTH
 */

/*
 * Determine a principle's "complete extent of access" to some object. This
 * is useful for implementing operations like the conventional UNIX access
 * function.  This routine implements a compatible subset of the DCE ACLs.
 *
 * The principle is represented by its PAC (pac) and the object is
 * represented by its ACL (ap).  The principle output parameter
 * (access_extent) is a mask containing a 1 for each permission granted by
 * first ACL Entry category (see below) of the ACL which the PAC matches, and
 * 0s elsewhere.  Note that the PAC may be NULL or unauthenticated.
 *
 * All ACL Entry types (sec_acl_e_type_*) are supported by this routine
 * and are in categories checked in the order shown below:
 *
 *	user,foreign_user	At most one match
 *	group,foreign_group	Union of all permissions for each matching group
 *	other_obj		Matching local realm accesses
 *	foreign_other		At most one can match
 *	any_other		Catch-all
 *	unauthenticated		(Note 1)
 *
 * Note: sec_acl_e_type_unauthenticated is a mask used for all matches on
 * unauthenticated PACs.  It is also intersected with sec_acl_e_type_any_other
 * for NULL PACs.
 *
 * NB:	Note that this routine does not care at all about the meaning of any
 *	particular permission bit.
 *
 * IMPLEMENTATION NOTES:
 *
 * Note that this routine does virtually no error checking in the interest of
 * speed.
 */

/* ______________________________________________________________________ */

/* Some notes on ACLE types - sec_acl_e_type_*, that is:
    These can only occur once in an ACL:
	user_obj group_obj other_obj
	mask_obj
	unauthenticated
	any_other
	user_obj_deleg group_obj_deleg other_obj_deleg
	any_other_deleg

    These can only occur myltiple times, and "take" a sec_id_t:
	user group foreign_other
	user_deleg group_deleg for_other_deleg

    These can only occur myltiple times, and "take" a sec_id_foreign_t:
	foreign_user foreign_group
	for_user_deleg for_group_deleg

    Only one user, foreign_other, user_deleg, for_other_deleg can match
    what's in the PAC, however.  Same for foreign_user, for_user_deleg.
*/

/* Determine and return the perms for unauthenticated user. */
static void
dce_acl__inq_unauthen_perms(
  /* [in] */
    sec_acl_t			*ap,
  /* [out] */
    sec_acl_permset_t		*perms
)
{
    sec_acl_entry_t	*any_other_ep, *unauth_ep, *other_obj_ep, *ep;
    sec_acl_permset_t	mask_perms, tmp1, tmp2;
    int			i;

    mask_perms = MAX_PERMISSIONS;
    any_other_ep = unauth_ep = other_obj_ep = NULL;
    for (i=ap->num_entries, ep=ap->sec_acl_entries; --i>=0; ep++) {
	switch (ep->entry_info.entry_type) {
	default:
	    break;
	case sec_acl_e_type_any_other:
	    any_other_ep = ep;
	    break;
	case sec_acl_e_type_unauthenticated:
	    unauth_ep = ep;
	    break;
	case sec_acl_e_type_other_obj:
	    /* This is here to be consistant with secd.  For a null
	     * PAC, other_obj is used without comparing the realm uuid */
	    other_obj_ep = ep;
	    break;
	case sec_acl_e_type_mask_obj:
	    mask_perms = ep->perms;
	    break;
	}
    }

/*
XXX - need to do something with delegation:  ???
	sec_acl_e_type_any_other_deleg
	sec_acl_e_type_other_obj_deleg
*/

    if (other_obj_ep && unauth_ep) {
	tmp1 = MASKIT(other_obj_ep, mask_perms);
	tmp2 = MASKIT(unauth_ep, mask_perms);
	*perms = tmp1 & tmp2;
    }
    else if (any_other_ep && unauth_ep) {
	tmp1 = MASKIT(any_other_ep, mask_perms);
	tmp2 = MASKIT(unauth_ep, mask_perms);
	*perms = tmp1 & tmp2;
    }
    else
	*perms = dce_acl_c_no_permissions;
    return;
}


/* The access algorithm.  Returns true if caller can just return (instead
 * of masking with unauthen perms).
 */
/* chagned from sec_id_pa_t to sec_id_pac_t - Mez */

static boolean
dce_acl__permset_alg(
  /* [in] */
    boolean			do_deleg,  /* currently ignored */
    boolean			auth,
    sec_id_pac_t		*pac_p,
    sec_acl_t			*ap,
    uuid_t			*own_id,
    uuid_t			*grp_id,
    sec_acl_permset_t		*mask_perms,
    sec_acl_permset_t		*unauth_mask,
    sec_acl_posix_semantics_t	posix_semantics,
  /* [out] */
    sec_acl_permset_t		*perms,
    error_status_t		*st
)
{
    boolean		chk_loc_groups, in_def_realm, one_group_found;
    boolean		check_groups;
    sec_acl_entry_t	*unauth_ep;
    sec_acl_entry_t	*user_obj_ep, *user_ep, *for_user_ep;
    sec_acl_entry_t	*other_obj_ep, *for_other_ep;
    sec_acl_entry_t	*any_other_ep;
    sec_acl_entry_t	*ep;
    sec_acl_permset_t	group_access;
    sec_acl_permset_t	tmp1;
    sec_id_t		*gp;
    uuid_t		*gr_uuid;
    sec_id_t		*realm;
    int			i, j;

    *st = error_status_ok;
    *perms = dce_acl_c_no_permissions;

    unauth_ep = user_obj_ep = user_ep = for_user_ep =
	other_obj_ep = for_other_ep = any_other_ep = NULL;

    /* We have a PAC.  Pre-process the acl entries so we don't have to loop
     * through the entries as many times.  Note, the accessor may be a member
     * of multiple [foreign_]group. Therefore, the group checks
     * must be done afterwords, in a separate pass through the entry list.
     */

    /* Set this just once now we know we have a PAC */
    in_def_realm = uuid_equal(&pac_p->realm.uuid, &ap->default_realm.uuid, st)
	|| uuid_is_nil(&ap->default_realm.uuid, st);

    check_groups = FALSE;
    for (i=ap->num_entries, ep=ap->sec_acl_entries; --i>=0; ep++) {

	/* Check for existence of each type of entry, and keep track
         * of where each type was found in the entry list. */
	switch (ep->entry_info.entry_type) {
	case sec_acl_e_type_user:
	    if (in_def_realm && uuid_equal(&pac_p->principal.uuid,
			&ep->entry_info.tagged_union.id.uuid, st))
		user_ep = ep;
	    break;
	case sec_acl_e_type_foreign_user:
	    if (uuid_equal(&pac_p->principal.uuid,
		    &ep->entry_info.tagged_union.foreign_id.id.uuid, st)
	     && SAME_FOREIGN_REALM(ep))
		for_user_ep = ep;
	    break;
	case sec_acl_e_type_other_obj:
	    if (in_def_realm)
		other_obj_ep = ep;
	    break;
	case sec_acl_e_type_foreign_other:
	    if (uuid_equal(&pac_p->realm.uuid,
		    &ep->entry_info.tagged_union.id.uuid, st))
		for_other_ep = ep;
	    break;
	case sec_acl_e_type_unauthenticated:
	    unauth_ep = ep;
	    break;
	case sec_acl_e_type_group:
	case sec_acl_e_type_foreign_group:
	case sec_acl_e_type_group_obj:
	    /* These are ignored by this pass over the acl.  Just flag that
	     * we should check groups later.
	     */
	    check_groups = TRUE;
	    break;
	case sec_acl_e_type_any_other:
	    any_other_ep = ep;
	    break;
	case sec_acl_e_type_mask_obj:
	    *mask_perms = ep->perms;
	    break;

	case sec_acl_e_type_user_obj:
	    user_obj_ep = ep;
	    break;
	    /* Unsupported ACL entry types. */
	default:
	    break;
	}

    }


    /* Now that we know which entries match the user described in the PAC,
     * check the permissions corresponding to each entry until access is
     * granted by one of them. */

    /* USER_OBJ check (masking is differnt than the others) */
    if (user_obj_ep) {
	/* If user_obj entry exists, owner cannot be NULL */
	if (own_id == NULL) {
	    *st = sec_acl_expected_user_obj;
	    return TRUE;
	}
	if (auth) {
	    if (uuid_equal(own_id, &pac_p->principal.uuid, st)) {
		*perms = user_obj_ep->perms;
		return TRUE;
	    }
	}
	else if (unauth_ep) {
	    *perms = user_obj_ep->perms & unauth_ep->perms;
	    return TRUE;
	}
    }

    /* USER check */
    if (user_ep) {
	*perms = MASKIT(user_ep, *mask_perms);
	return FALSE;
    }

    /* FOREIGN_USER check */
    if (for_user_ep) {
	*perms = MASKIT(for_user_ep, *mask_perms);
	return FALSE;
    }

    /* GROUP checks */
    one_group_found = FALSE;
    if (check_groups) {
	sec_acl_entry_type_t	e_type;
	group_access = dce_acl_c_no_permissions;
	chk_loc_groups  = FALSE;
	for (i=ap->num_entries, ep=ap->sec_acl_entries; --i>=0; ep++) {
	    e_type = ep->entry_info.entry_type;
	    switch (e_type) {
	    default:
		break;
	    case sec_acl_e_type_group_obj:
	    case sec_acl_e_type_group:
	    case sec_acl_e_type_foreign_group:
		/* GROUP_OBJ */
		if (e_type == sec_acl_e_type_group_obj ) {
		    /* If group_obj entry exists then group_obj can't be NULL */
		    if (grp_id == NULL) {
			*st = sec_acl_expected_group_obj;
			return FALSE;
		    }
		    chk_loc_groups = in_def_realm;
		    gr_uuid = grp_id;
		    realm   = &ap->default_realm;
		}
		/* GROUP */
		else if (e_type == sec_acl_e_type_group) {
		    chk_loc_groups = in_def_realm;
		    gr_uuid = &ep->entry_info.tagged_union.id.uuid;
		    realm   = &ap->default_realm;
		}
		/* FOREIGN_GROUP */
		else {		/* foreign groups */
		    chk_loc_groups = SAME_FOREIGN_REALM(ep);
		    gr_uuid = &ep->entry_info.tagged_union.foreign_id.id.uuid;
		    realm   = &ep->entry_info.tagged_union.foreign_id.realm;
		}

		/* Check either the local groups or the foreign groups */
		if (chk_loc_groups) {
		    if (uuid_equal(&pac_p->group.uuid, gr_uuid, st)) {
			one_group_found = TRUE;
			tmp1 = MASKIT(ep, *mask_perms);
			group_access |= tmp1;
		    }

		    /* CHECK LOCAL GROUPS */
		    for (gp=pac_p->groups, j=pac_p->num_groups; --j>=0; gp++)
			if (uuid_equal(&gp->uuid, gr_uuid, st)) {
			    one_group_found = TRUE;
			    tmp1 = MASKIT(ep, *mask_perms);
			    group_access |= tmp1;
			}
		}
		else {
		    int	fg_i, fgs_i;
/* changed to v103 foreign groups -  Mez */
		    sec_id_foreign_t *fgs;

		    /* CHECK FOREIGN GROUPS (which are in groupsets) */
		    fgs = pac_p->foreign_groups;
		    for (fgs_i=pac_p->num_foreign_groups; --fgs_i>=0; fgs++) {
			 if (!uuid_equal(&gp->uuid, &realm->uuid, st))
			    continue;
			if (uuid_equal(&fgs->id.uuid, gr_uuid, st)) {
			    one_group_found = TRUE;
			    tmp1 = MASKIT(ep, *mask_perms);
			    group_access |= tmp1;
			    }
		    }
		}
		break;
	    }
	}
    }

    /* If one or more groups found then we take the union of all matching
     * group's permissions and call that the access extent. */
    if (one_group_found) {
	*perms = group_access;
	return FALSE;
    }

    /* OTHER_OBJ check */
    if (other_obj_ep) {
	*perms = MASKIT(other_obj_ep, *mask_perms);
	return FALSE;
    }

    /* FOREIGN_OTHER check */
    if (for_other_ep) {
	*perms = MASKIT(for_other_ep, *mask_perms);
	return FALSE;
    }

    /* ANY_OTHER check */
    if (any_other_ep) {
	*perms = MASKIT(any_other_ep, *mask_perms);
	return FALSE;
    }

    /* No ACL entry gave any access. */
    *perms = dce_acl_c_no_permissions;
    return TRUE;
}


/* Was dce_acl_inq_permset_for_creds - changed for pac. Mez */

void
dce_acl_inq_permset_for_pac(
  /* [in] */
    sec_id_pac_t	        *pac_p,
    sec_acl_t			*ap,
    uuid_t			*own_id,
    uuid_t			*grp_id,
    sec_acl_posix_semantics_t	posix_semantics,
  /* [out] */
    sec_acl_permset_t		*perms,
    error_status_t		*st
)
{
    boolean			auth, ret;
    sec_acl_permset_t		mask_perms, unauth_mask;
    error_status_t		xst;

    *st = error_status_ok;
    *perms = dce_acl_c_no_permissions;

    /* If the mask isn't explicitly set, then it should have no effect when
     * masking, so turn all perm bits on by default. */
    mask_perms  = MAX_PERMISSIONS;
    unauth_mask = 0;

    auth = FALSE;
    if (dce_acl_is_unauthenticated(pac_p)) {
	dce_acl__inq_unauthen_perms(ap, perms);
	return;
    }
    else 
      auth = TRUE;

    if (dce_acl__permset_alg(FALSE, auth, pac_p, ap, own_id, grp_id,
		&mask_perms, &unauth_mask, posix_semantics, perms, st) ) {
	return;
    }

    if (!auth) {
	if (unauth_mask == 0)
	    *perms = dce_acl_c_no_permissions;
	else {
	    *perms = mask_perms & unauth_mask;
	}
    }
}
