/*
 * Copyright (c) 1995: Open Software Foundation Inc.
 *
 * All Rights Reserved. Please see COPYRIGHT.DCEWEB
 * in the top-level directory of this release.
 */
/* ______________________________________________________________________ */
/*
 * @OSF_COPYRIGHT@
 * COPYRIGHT NOTICE
 * Copyright (c) 1993 Open Software Foundation, Inc.
 * ALL RIGHTS RESERVED (DCE).  See the file named COPYRIGHT.DCE in the
 * src directory for the full copyright text.
 */
/*
 * HISTORY
 * $Log: dblock.c,v $
 * Revision 1.1.2.3  1994/09/16  20:22:18  bowe
 * 	Use app_mutex, not mutex. [CR 12231]
 * 	[1994/09/16  20:20:25  bowe]
 *
 * Revision 1.1.2.2  1994/03/29  21:04:33  bowe
 * 	Added error checking for lock/unlock
 * 	[1994/03/29  21:03:17  bowe]
 * 
 * Revision 1.1.2.1  1994/01/24  22:01:43  bowe
 * 	Initial checkin
 * 	[1994/01/24  21:59:33  bowe]
 * 
 * Revision 1.1.2.1  1993/12/02  20:46:12  bowe
 * 	Initial revision.
 * 	[1993/12/02  20:44:50  bowe]
 * 
 * $EndLog$
 */

/* ______________________________________________________________________ 
 *
 *  Database / backing store locking routines
 *  External interfaces:
 *	dce_db_lock()
 *	dce_db_unlock()
 */

#include <stdio.h>

#include <dce/uuid.h>
#include <dbimpl.h>


void
dce_db_lock(
    dce_db_handle_t	h,
    error_status_t	*st
)
{
    *st = error_status_ok;
    if (pthread_mutex_lock(&h->app_mutex) == -1) {
	if (errno == EDEADLK)
	    *st = db_s_already_locked;
	else
	    *st = db_s_mutex_lock_fail;
    }
}

void
dce_db_unlock(
    dce_db_handle_t	h,
    error_status_t	*st
)
{
    *st = error_status_ok;
    if (pthread_mutex_unlock(&h->app_mutex) == -1) {
	if (errno == EINVAL)
	    *st = db_s_not_locked;
	else
	    *st = db_s_mutex_lock_fail;
    }
}

